@@ -1,6 +1,7 @@
 """Visp geometric calibration task."""
 import logging
 from typing import Generator
+from typing import Optional
 
 import numpy as np
 import peakutils as pku
@@ -12,7 +13,10 @@ from dkist_processing_math.feature import find_px_angles
 from dkist_processing_math.statistics import average_numpy_arrays
 from dkist_processing_math.transform import do_hough
 from dkist_processing_math.transform import make_binary
+from scipy.fft import fftn
+from scipy.fft import ifftn
 from scipy.optimize import minimize
+from skimage.registration._phase_cross_correlation import _upsampled_dft
 
 from dkist_processing_visp.models.tags import VispTag
 from dkist_processing_visp.tasks.mixin.corrections import CorrectionsMixin
@@ -20,6 +24,7 @@ from dkist_processing_visp.tasks.mixin.input_frame_loaders import InputFrameLoad
 from dkist_processing_visp.tasks.mixin.intermediate_frame_helpers import (
     IntermediateFrameHelpersMixin,
 )
+from dkist_processing_visp.tasks.solar import SolarCalibration
 from dkist_processing_visp.tasks.visp_base import VispTaskBase
 
 
@@ -77,6 +82,7 @@ class GeometricCalibration(
             with self.apm_task_step(f"Generating geometric calibrations for {beam = }"):
                 with self.apm_processing_step(f"Computing and writing angle for {beam = }"):
                     logging.info(f"Compute and write angle for {beam = }")
+                    # TODO: Why not use Lamp gains for angle measurement? They don't have strong spectral lines.
                     angle = self.compute_beam_angle(beam=beam)
                     self.write_angle(angle=angle, beam=beam)
 
@@ -132,6 +138,7 @@ class GeometricCalibration(
         """Run before run() with Elastic APM span capturing."""
         super().pre_run()
         self._fiducial_array = None
+        self._fiducial_mask = None
 
     def basic_corrected_data(self, beam: int, modstate: int) -> np.ndarray:
         """
@@ -177,6 +184,32 @@ class GeometricCalibration(
         """
         self._fiducial_array = array
 
+    @property
+    def fiducial_mask(self) -> np.ndarray:
+        """Mask on target array used for determining state offsets.
+
+        Pixels NOT around strong lines are masked out.
+        """
+        if self._fiducial_mask is None:
+            raise ValueError("Fiducial array has not been set. This should never happen.")
+        return self._fiducial_mask
+
+    @fiducial_mask.setter
+    def fiducial_mask(self, array: np.ndarray) -> None:
+        """
+        Set the target mask used for determining state offsets.
+
+        Parameters
+        ----------
+        array : np.ndarray
+            Fiducial mask
+
+        Returns
+        -------
+        None
+        """
+        self._fiducial_mask = array
+
     def offset_corrected_array_generator(self, beam: int) -> Generator[np.ndarray, None, None]:
         """
         All modstates for a single beam that have had their state offset applied.
@@ -291,7 +324,6 @@ class GeometricCalibration(
         np.ndarray
             Array corrected for beam angle
         """
-        logging.info(f"Removing beam angle from {beam=}, {modstate=}")
         # TODO: Figure out why the type of basic_corrected_data was >f8 prior to conversion
         beam_mod_array = self.basic_corrected_data(beam=beam, modstate=modstate).astype(np.float64)
         corrected_array = next(self.corrections_correct_geometry(beam_mod_array, angle=angle))
@@ -320,14 +352,20 @@ class GeometricCalibration(
         if beam == 1 and modstate == 1:
             logging.info("Set fiducial array")
             self.fiducial_array = array
+            self.fiducial_mask = self.compute_offset_mask(array)
             return np.zeros(2)
 
+        array_mask = self.compute_offset_mask(array)
         shift = self.compute_single_state_offset(
             fiducial_array=self.fiducial_array,
             array=array,
             upsample_factor=self.parameters.geo_upsample_factor,
+            fiducial_mask=self.fiducial_mask,
+            array_mask=array_mask,
+        )
+        logging.info(
+            f"Offset for {beam = } and {modstate = } is {np.array2string(shift, precision=3)}"
         )
-        logging.info(f"Offset for {beam=} and {modstate=} is {np.array2string(shift, precision=3)}")
 
         return shift
 
@@ -389,6 +427,10 @@ class GeometricCalibration(
         avg_beam_array = average_numpy_arrays(beam_generator)
         num_spec = avg_beam_array.shape[1]
 
+        self.intermediate_frame_helpers_write_arrays(
+            arrays=avg_beam_array, beam=beam, task="DEBUG_GC_AVG_OFFSET"
+        )
+
         ref_spec = avg_beam_array[:, num_spec // 2]
         beam_shifts = np.empty(num_spec) * np.nan
         for j in range(num_spec):
@@ -451,7 +493,11 @@ class GeometricCalibration(
 
     @staticmethod
     def compute_single_state_offset(
-        fiducial_array: np.ndarray, array: np.ndarray, upsample_factor: float = 1000.0
+        fiducial_array: np.ndarray,
+        array: np.ndarray,
+        upsample_factor: float = 1000.0,
+        fiducial_mask: Optional[np.ndarray] = None,
+        array_mask: Optional[np.ndarray] = None,
     ) -> np.ndarray:
         """
         Find the (x, y) shift between the current beam and the reference beam.
@@ -471,16 +517,82 @@ class GeometricCalibration(
         numpy.ndarray
             The (x, y) shift between the reference beam and the current beam at hand
         """
-        # Pixel precision is 1/upsample factor
         shift = skir.phase_cross_correlation(
-            fiducial_array, array, return_error=False, upsample_factor=upsample_factor
+            fiducial_array,
+            array,
+            return_error=False,
+            reference_mask=fiducial_mask,
+            moving_mask=array_mask,
         )
 
+        if upsample_factor != 1.0:
+            logging.info(f"Rough shift = {np.array2string(-1 * shift, precision=3)}")
+            shift = GeometricCalibration.refine_modstate_shift(
+                shift, fiducial_array, array, upsample_factor=upsample_factor
+            )
+
         # Multiply by -1 so that the output is the shift needed to move from "perfect" to the current state.
         #  In other words, applying a shift equal to the negative of the output of this function will undo the measured
         #  shift.
         return -shift
 
+    @staticmethod
+    def refine_modstate_shift(
+        shifts: np.ndarray,
+        fiducial_array: np.ndarray,
+        target_array: np.ndarray,
+        upsample_factor: float = 10.0,
+    ):
+        """Refine a shift from `compute_single_state_offset` using an upsampling technique.
+
+        This is taken directly from the un-masked version of `skimage.registration.phase_cross_correlation`.
+        """
+        # All comments below are copied from `skimage.registration.phase_cross_correlation`
+        src_freq = fftn(fiducial_array)
+        target_freq = fftn(target_array)
+        image_product = src_freq * target_freq.conj()
+
+        upsample_factor = np.array(upsample_factor, dtype=fiducial_array.dtype)
+        shifts = np.round(shifts * upsample_factor) / upsample_factor
+        upsampled_region_size = np.ceil(upsample_factor * 1.5)
+        # Center of output array at dftshift + 1
+        dftshift = np.fix(upsampled_region_size / 2.0)
+        # Matrix multiply DFT around the current shift estimate
+        sample_region_offset = dftshift - shifts * upsample_factor
+        cross_correlation = _upsampled_dft(
+            image_product.conj(), upsampled_region_size, upsample_factor, sample_region_offset
+        ).conj()
+        # Locate maximum and map back to original pixel grid
+        maxima = np.unravel_index(np.argmax(np.abs(cross_correlation)), cross_correlation.shape)
+
+        maxima = np.stack(maxima).astype(fiducial_array.dtype, copy=False)
+        maxima -= dftshift
+
+        shifts += maxima / upsample_factor
+
+        return shifts
+
+    def compute_offset_mask(self, array: np.ndarray) -> np.ndarray:
+        """Generate a 2D mask that exclude non-line regions.
+
+        These masks are then used to greatly improve the accuracy of the phase cross-correlation used in
+        `compute_single_state_offset`.
+        """
+        zone_kwargs = {
+            "prominence": self.parameters.solar_zone_prominence,
+            "width": self.parameters.solar_zone_width,
+            "bg_order": self.parameters.solar_zone_bg_order,
+            "normalization_percentile": self.parameters.solar_zone_normalization_percentile,
+            "rel_height": self.parameters.solar_zone_rel_height,
+        }
+        zones = SolarCalibration.compute_line_zones(array, **zone_kwargs)
+        logging.info(f"Found {zones = }")
+        mask = np.zeros(array.shape).astype(bool)
+        for z in zones:
+            mask[z[0] : z[1], :] = True
+
+        return mask
+
     def find_brightest_modstate(self, beam: int) -> np.ndarray:
         """
         Return the array for the modstate with the largest median value.
