@@ -107,8 +107,12 @@ class InstrumentPolarizationCalibration(
                 )
 
             with self.apm_processing_step("Computing and recording polcal quality metrics"):
+                logging.info("Computing and storing PolCal quality metrics")
                 self.record_polcal_quality_metrics(beam, polcal_fitter=pac_fitter)
 
+            with self.apm_processing_step("Computing and saving intermediate polcal files"):
+                self.save_intermediate_polcal_files(beam, polcal_fitter=pac_fitter)
+
         with self.apm_processing_step("Computing and logging quality metrics"):
             no_of_raw_polcal_frames: int = self.scratch.count_all(
                 tags=[
@@ -259,6 +263,8 @@ class InstrumentPolarizationCalibration(
                 self.parameters.polcal_num_spectral_bins,
                 self.parameters.polcal_num_spatial_bins,
             )
+            # We don't use resize_arrays_local_mean here because the hairlines massively impact the signal in their
+            # beams. Maybe in the future we could clean the hairlines??
             local_binned_array = next(resize_arrays(spectral_corrected_array, output_shape))
             global_binned_array = next(resize_arrays(spectral_corrected_array, (1, 1)))
 
@@ -315,16 +321,26 @@ class InstrumentPolarizationCalibration(
         logging.info(f"Target full-frame demodulation shape: {target_shape}")
         return self._resize_polcal_array(sliced_demod, target_shape)
 
-    # TODO: Put more/real stuff here
-    # TODO: Add test coverage?
-    def record_polcal_quality_metrics(
+    # TODO: Add test coverage? (See note in *-common.tasks.mixin.quality._metrics._PolcalQualityMixin
+    def record_polcal_quality_metrics(self, beam: int, polcal_fitter: PolcalFitter):
+        """Record various quality metrics from PolCal fits."""
+        self.quality_store_polcal_results(
+            polcal_fitter=polcal_fitter,
+            label=f"Beam {beam}",
+            bins_1=self.parameters.polcal_num_spectral_bins,
+            bins_2=self.parameters.polcal_num_spatial_bins,
+            bin_1_type="spectral",
+            bin_2_type="spatial",
+        )
+
+    def save_intermediate_polcal_files(
         self,
         beam: int,
         polcal_fitter: PolcalFitter,
     ) -> None:
-        """Record various quality metrics from PolCal fits.
+        """Save intermediate files for science-team analysis.
 
-        THIS FUNCTION IS INCOMPLETE. Right now it just saves some files for manual inspection.
+        THIS FUNCTION IS ONLY TEMPORARY. It should probably be removed prior to production.
         """
         dresser = polcal_fitter.local_objects.dresser
         ## Input flux
@@ -410,4 +426,6 @@ class InstrumentPolarizationCalibration(
         self.single_beam_shape = array.shape
 
     def _resize_polcal_array(self, array: np.ndarray, output_shape: Tuple[int, ...]) -> np.ndarray:
-        return next(resize_arrays(array, output_shape))
+        return next(
+            resize_arrays(array, output_shape, order=self.parameters.polcal_demod_upsample_order)
+        )
