@@ -1,6 +1,7 @@
 """Visp write L1 task."""
 import logging
 import uuid
+from datetime import datetime
 from typing import Literal
 
 import numpy as np
@@ -55,17 +56,19 @@ class VispWriteL1Frame(WriteL1Frame):
 
         if stokes.upper() not in self.constants.stokes_params:
             raise ValueError("The stokes parameter must be one of I, Q, U, V")
-        # ---Spectral---
-        header["DNAXIS1"] = header["NAXIS1"]
-        header["DTYPE1"] = "SPECTRAL"
-        header["DPNAME1"] = "dispersion axis"
-        header["DWNAME1"] = "wavelength"
-        header["DUNIT1"] = header["CUNIT1"]
+
         # ---Spatial 1---
+        header["DNAXIS1"] = header["NAXIS1"]
+        header["DTYPE1"] = "SPATIAL"
+        header["DPNAME1"] = "spatial along slit"
+        header["DWNAME1"] = "helioprojective latitude"
+        header["DUNIT1"] = header["CUNIT1"]
+
+        # ---Spectral---
         header["DNAXIS2"] = header["NAXIS2"]
-        header["DTYPE2"] = "SPATIAL"
-        header["DPNAME2"] = "spatial along slit"
-        header["DWNAME2"] = "helioprojective latitude"
+        header["DTYPE2"] = "SPECTRAL"
+        header["DPNAME2"] = "dispersion axis"
+        header["DWNAME2"] = "wavelength"
         header["DUNIT2"] = header["CUNIT2"]
 
         # ---Spatial 2---
@@ -89,7 +92,7 @@ class VispWriteL1Frame(WriteL1Frame):
                 f"DNAXIS{num_axis}"
             ] = self.constants.num_map_scans  # total number of raster scans in the dataset
             header[f"DTYPE{num_axis}"] = "TEMPORAL"
-            header[f"DPNAME{num_axis}"] = "time"
+            header[f"DPNAME{num_axis}"] = "raster map repeat number"
             header[f"DWNAME{num_axis}"] = "time"
             header[f"DUNIT{num_axis}"] = "s"
             # Temporal position in dataset
@@ -132,8 +135,7 @@ class VispWriteL1Frame(WriteL1Frame):
 
         return header
 
-    @staticmethod
-    def _replace_header_values(header: fits.Header, data: np.ndarray) -> fits.Header:
+    def _replace_header_values(self, header: fits.Header, data: np.ndarray) -> fits.Header:
         """
         Replace the FILE_ID and DATE keywords with new values.
 
@@ -141,10 +143,54 @@ class VispWriteL1Frame(WriteL1Frame):
         and the *-common version of this method would overwrite that key.
         """
         header["FILE_ID"] = uuid.uuid4().hex
-        header["DATE"] = Time.now().fits
+        header["DATE"] = Time(datetime.now().isoformat(), precision=6).fits
         # Remove BZERO and BSCALE as their value should be recalculated by astropy upon fits write
         header.pop("BZERO", None)
         header.pop("BSCALE", None)
         # Make sure that NAXIS is set to the shape of the data in case of squeezing
         header["NAXIS"] = len(data.shape)
+        # Check that the WCS headers are in the right order
+        if header["CTYPE1"] == "AWAV":  # Then the order is wrong
+            header = self.transpose_wcs_keys(header=header)
+        return header
+
+    @staticmethod
+    def transpose_wcs_keys(header: fits.Header) -> fits.Header:
+        """
+        In early DKIST operations, some WCS keys and the PCi_j matrix were incorrectly ordered.
+
+        This function transposes them to reverse it.
+        """
+        logging.info(f"Transposing axes 1 and 2 of the WCS headers")
+
+        # What keys need to be transposed? Substitute "n" for the axis number.
+        # All FITS keywords are supposed to be uppercase so using lowercase "n" should be OK
+        # CRPIXn is excluded from this list as it is observed to be in the
+        # correct order already (does not need transposing.)
+        key_list = [
+            "CRDATEn",
+            "CRVALn",
+            "CDELTn",
+            "CUNITn",
+            "CTYPEn",
+        ]
+
+        # Gather the values from the key_list keywords and swap them in place
+        for key in key_list:
+            for suffix in ["", "A"]:
+                key_1 = f"{key}{suffix}".replace("n", "1")
+                key_2 = f"{key}{suffix}".replace("n", "2")
+                try:
+                    header[key_1], header[key_2] = header[key_2], header[key_1]
+                except KeyError:  # The "A" variant of keywords are not required
+                    logging.info(
+                        f"Keywords {key_1} and {key_2} were not switched as they do not exist in this frame."
+                    )
+
+        # The PC matrix is observed to be in a different order.
+        # It is in the lt, ln, wave order, when we want it in the lt, wave, ln order.
+        for n in range(1, 4):
+            header[f"PC2_{n}"], header[f"PC3_{n}"] = header[f"PC3_{n}"], header[f"PC2_{n}"]
+        # Note also that the reordered value of PC2_2 (i.e. wave) is probably wrong and should be one
+
         return header
