@@ -14,6 +14,7 @@ from unittest.mock import PropertyMock
 import numpy as np
 import pytest
 from astropy.io import fits
+from astropy.wcs import WCS
 from dkist_data_simulator.dataset import key_function
 from dkist_data_simulator.spec122 import Spec122Dataset
 from dkist_header_validator import spec122_validator
@@ -57,6 +58,17 @@ class VispHeaders(Spec122Dataset):
     def date(self, key: str):
         return choice([1, 2])
 
+    @property
+    def fits_wcs(self):
+        w = WCS(naxis=self.array_ndim)
+        w.wcs.crpix = self.array_shape[2] / 2, self.array_shape[1] / 2, 1
+        w.wcs.crval = 0, 0, 0
+        w.wcs.cdelt = 1, 1, 1
+        w.wcs.cunit = "arcsec", "nm", "arcsec"
+        w.wcs.ctype = "HPLT-TAN", "AWAV", "HPLN-TAN"
+        w.wcs.pc = np.identity(self.array_ndim)
+        return w
+
 
 class VispHeadersValidDarkFrames(VispHeaders):
     def __init__(
@@ -345,7 +357,7 @@ def calibrated_visp_dataset(visp_dataset) -> fits.Header:
 
     Because now that's added during ScienceCal
     """
-    visp_dataset["DATE-END"] = "2000-01-01T12:00:00"
+    visp_dataset["DATE-END"] = "2000-01-01T12:00:00.000000"
 
     return visp_dataset
 
@@ -363,7 +375,7 @@ class VispTestingParameters:
     visp_geo_num_theta: int = 1500
     visp_geo_theta_min: float = -np.pi / 90
     visp_geo_theta_max: float = np.pi / 90
-    visp_geo_upsample_factor: float = 1000.0
+    visp_geo_upsample_factor: float = 10.0
     visp_geo_max_shift: float = 40.0
     visp_geo_poly_fit_order: int = 3
     visp_solar_spectral_avg_window: int = 50
@@ -380,6 +392,7 @@ class VispTestingParameters:
     visp_max_cs_step_time_sec: float = 20.0
     visp_polcal_num_spatial_bins: int = 3
     visp_polcal_num_spectral_bins: int = 1
+    visp_polcal_demod_upsample_order: int = 3
     visp_pac_fit_mode: str = "use_M12"
     visp_pac_init_set: str = "OCCal_VIS"
 
