@@ -1,5 +1,5 @@
 import json
-from unittest.mock import PropertyMock
+from dataclasses import dataclass
 
 import numpy as np
 import pytest
@@ -8,16 +8,31 @@ from dkist_header_validator import spec122_validator
 from dkist_processing_common._util.scratch import WorkflowFileSystem
 from dkist_processing_common.models.tags import Tag
 
-from dkist_processing_visp.models.constants import VispBudName
 from dkist_processing_visp.models.tags import VispTag
 from dkist_processing_visp.tasks.dark import DarkCalibration
 from dkist_processing_visp.tests.conftest import FakeGQLClient
 from dkist_processing_visp.tests.conftest import generate_fits_frame
+from dkist_processing_visp.tests.conftest import VispConstantsDb
 from dkist_processing_visp.tests.conftest import VispHeadersValidDarkFrames
+from dkist_processing_visp.tests.conftest import VispTestingParameters
+
+
+@dataclass
+class VispDarkTestingParameters(VispTestingParameters):
+    visp_beam_border: int = 10
 
 
 @pytest.fixture(scope="function")
-def dark_calibration_task(tmp_path, recipe_run_id, mocker):
+def dark_calibration_task(
+    tmp_path, assign_input_dataset_doc_to_task, init_visp_constants_db, recipe_run_id
+):
+    constants_db = VispConstantsDb(
+        LAMP_EXPOSURE_TIMES=(100.0,),
+        SOLAR_EXPOSURE_TIMES=(1.0,),
+        OBSERVE_EXPOSURE_TIMES=(0.01,),
+        POLCAL_EXPOSURE_TIMES=(),
+    )
+    init_visp_constants_db(recipe_run_id, constants_db)
     with DarkCalibration(
         recipe_run_id=recipe_run_id, workflow_name="dark_calibration", workflow_version="VX.Y"
     ) as task:
@@ -26,50 +41,49 @@ def dark_calibration_task(tmp_path, recipe_run_id, mocker):
         unused_time = 200.0
         num_exp_time = len(exp_times)
         num_frames_per = 3
-        task.scratch = WorkflowFileSystem(scratch_base_path=tmp_path, recipe_run_id=recipe_run_id)
-        mocker.patch(
-            "dkist_processing_visp.visp_base.VispScienceTask.correct_for_polarization",
-            new_callable=PropertyMock,
-            return_value=True,
-        )
-        task.constants.update(
-            {
-                VispBudName.lamp_exposure_times.value: [100.0],
-                VispBudName.solar_exposure_times.value: [1.0],
-                VispBudName.observe_exposure_times.value: [0.01],
-                VispBudName.polcal_exposure_times.value: [],
-            }
-        )
-        ds = VispHeadersValidDarkFrames(
-            dataset_shape=(num_beam * (num_exp_time + 1) * num_frames_per, 10, 10),
-            array_shape=(1, 10, 10),
-            time_delta=10,
-        )
-        header_generator = (
-            spec122_validator.validate_and_translate_to_214_l0(
-                d.header(), return_type=fits.HDUList
-            )[0].header
-            for d in ds
-        )
-        for b in range(num_beam):
+        array_shape = (1, 20, 10)
+        dataset_shape = ((num_exp_time + 1) * num_frames_per, 20, 10)
+        try:  # This try... block is here to make sure the dbs get cleaned up if there's a failure in the fixture
+            task.scratch = WorkflowFileSystem(
+                scratch_base_path=tmp_path, recipe_run_id=recipe_run_id
+            )
+            assign_input_dataset_doc_to_task(task, VispDarkTestingParameters())
+            beam_border = task.parameters.beam_border
+            ds = VispHeadersValidDarkFrames(
+                dataset_shape=dataset_shape,
+                array_shape=array_shape,
+                time_delta=10,
+            )
+            header_generator = (
+                spec122_validator.validate_and_translate_to_214_l0(
+                    d.header(), return_type=fits.HDUList
+                )[0].header
+                for d in ds
+            )
             for e in exp_times + [unused_time]:  # Make some darks we won't use
                 for _ in range(num_frames_per):
-                    hdul = generate_fits_frame(header_generator=header_generator)
+                    hdul = generate_fits_frame(header_generator=header_generator, shape=array_shape)
                     hdul[0].data *= 0
-                    hdul[0].data += (b + 1) * e
+                    # Create combined 2-beam array
+                    beam = 1
+                    hdul[0].data[0, :beam_border, :] = beam * e
+                    beam = 2
+                    hdul[0].data[0, beam_border:, :] = beam * e
                     task.fits_data_write(
                         hdu_list=hdul,
                         tags=[
                             VispTag.input(),
                             VispTag.frame(),
                             VispTag.task("DARK"),
-                            VispTag.beam(b + 1),
                             VispTag.exposure_time(e),
                         ],
                     )
-        yield task, num_beam, exp_times, unused_time
-        task.scratch.purge()
-        task.constants.purge()
+            yield task, num_beam, exp_times, unused_time
+        except:
+            raise
+        finally:
+            task.scratch.purge()
+            task.constants._purge()
 
 
 def test_dark_calibration_task(dark_calibration_task, mocker):
@@ -119,6 +133,7 @@ def test_dark_calibration_task(dark_calibration_task, mocker):
         with file.open() as f:
             data = json.load(f)
             assert isinstance(data, dict)
-            assert data["total_frames"] == task.count(
+            assert data["total_frames"] == task.scratch.count_all(
                 tags=[VispTag.input(), VispTag.frame(), VispTag.task("DARK")]
             )
+            assert data["frames_not_used"] == 3
