@@ -1,4 +1,5 @@
 import json
+from dataclasses import dataclass
 from datetime import datetime
 
 import numpy as np
@@ -6,55 +7,62 @@ import pytest
 from astropy.io import fits
 from dkist_header_validator import spec122_validator
 from dkist_processing_common._util.scratch import WorkflowFileSystem
-from dkist_processing_common.models.constants import BudName
 from dkist_processing_common.models.tags import Tag
 
-from dkist_processing_visp.models.constants import VispBudName
 from dkist_processing_visp.models.tags import VispTag
 from dkist_processing_visp.tasks.lamp import LampCalibration
 from dkist_processing_visp.tests.conftest import FakeGQLClient
 from dkist_processing_visp.tests.conftest import generate_fits_frame
+from dkist_processing_visp.tests.conftest import VispConstantsDb
 from dkist_processing_visp.tests.conftest import VispHeadersValidLampGainFrames
+from dkist_processing_visp.tests.conftest import VispTestingParameters
 
 RNG = np.random.default_rng()
 
 
+@dataclass
+class VispLampTestingParameters(VispTestingParameters):
+    visp_beam_border: int = 10
+
+
 @pytest.fixture(scope="function")
-def lamp_calibration_task(tmp_path, recipe_run_id, input_dataset_document_with_simple_parameters):
+def lamp_calibration_task(
+    tmp_path, recipe_run_id, assign_input_dataset_doc_to_task, init_visp_constants_db, mocker
+):
     number_of_modstates = 2
     number_of_beams = 2
     exposure_time = 10.0  # From VispHeadersValidLampGainFrames fixture
+    dataset_shape = (number_of_modstates, 20, 10)
+    array_shape = (1, 20, 10)
+    intermediate_shape = (10, 10)
+    constants_db = VispConstantsDb(
+        NUM_MODSTATES=number_of_modstates, LAMP_EXPOSURE_TIMES=(exposure_time,)
+    )
+    init_visp_constants_db(recipe_run_id, constants_db)
     with LampCalibration(
         recipe_run_id=recipe_run_id, workflow_name="lamp_gain_calibration", workflow_version="VX.Y"
     ) as task:
-        task.scratch = WorkflowFileSystem(scratch_base_path=tmp_path, recipe_run_id=recipe_run_id)
-        doc_path = task.scratch.workflow_base_path / "dataset_doc.json"
-        with open(doc_path, "w") as f:
-            f.write(json.dumps(input_dataset_document_with_simple_parameters))
-        task.tag(doc_path, VispTag.input_dataset())
-        task.constants[BudName.num_modstates.value] = number_of_modstates
-        task.constants[VispBudName.lamp_exposure_times.value] = [exposure_time]
-
-        dark_signal = 3.0
-        start_time = datetime.now()
-        for j in range(number_of_beams):
-            # Make intermediate dark frame
-            dark_cal = np.ones((10, 10)) * dark_signal
-            dark_hdul = fits.HDUList([fits.PrimaryHDU(data=dark_cal)])
-            task.fits_data_write(
-                hdu_list=dark_hdul,
-                tags=[
-                    VispTag.intermediate(),
-                    VispTag.frame(),
-                    VispTag.task("DARK"),
-                    VispTag.beam(j + 1),
-                    VispTag.exposure_time(exposure_time),
-                ],
+        try:  # This try... block is here to make sure the dbs get cleaned up if there's a failure in the fixture
+            task.scratch = WorkflowFileSystem(
+                scratch_base_path=tmp_path, recipe_run_id=recipe_run_id
             )
+            assign_input_dataset_doc_to_task(task, VispLampTestingParameters())
+            beam_border = task.parameters.beam_border
+            dark_signal = 3.0
+            start_time = datetime.now()
+            # Make intermediate dark frame
+            dark_cal = np.ones(intermediate_shape) * dark_signal
+            dark_hdul = fits.HDUList([fits.PrimaryHDU(data=dark_cal)])
+            # Need a dark for each beam
+            for b in range(number_of_beams):
+                task.intermediate_frame_helpers_write_arrays(
+                    arrays=dark_cal, beam=b + 1, task="DARK", exposure_time=exposure_time
+                )
+            # These images are for two combined beams
             for i in range(number_of_modstates):
                 ds = VispHeadersValidLampGainFrames(
-                    dataset_shape=(number_of_modstates, 10, 10),
-                    array_shape=(1, 10, 10),
+                    dataset_shape=dataset_shape,
+                    array_shape=array_shape,
                     time_delta=10,
                     num_modstates=number_of_modstates,
                     modstate=i + 1,
@@ -66,8 +74,11 @@ def lamp_calibration_task(tmp_path, recipe_run_id, input_dataset_document_with_s
                     )[0].header
                     for d in ds
                 )
-                hdul = generate_fits_frame(header_generator=header_generator)
-                hdul[0].data = np.ones((10, 10)) + i + j + dark_signal
+                hdul = generate_fits_frame(header_generator=header_generator, shape=array_shape)
+                # Tweak data so that beam sides are slightly different
+                # Use data != 1 to check normalization in test
+                hdul[0].data[0, :beam_border, :] = 1.1
+                hdul[0].data[0, beam_border:, :] = 1.2
                 task.fits_data_write(
                     hdu_list=hdul,
                     tags=[
@@ -75,13 +86,15 @@ def lamp_calibration_task(tmp_path, recipe_run_id, input_dataset_document_with_s
                         VispTag.task("LAMP_GAIN"),
                         VispTag.modstate(i + 1),
                         VispTag.frame(),
-                        VispTag.beam(j + 1),
                         VispTag.exposure_time(exposure_time),
                     ],
                 )
-        yield task, number_of_modstates, number_of_beams
-        task.scratch.purge()
-        task.constants.purge()
+            yield task, number_of_modstates, number_of_beams
+        except:
+            raise
+        finally:
+            task.scratch.purge()
+            task.constants._purge()
 
 
 def test_lamp_calibration_task(lamp_calibration_task, mocker):
@@ -114,7 +127,7 @@ def test_lamp_calibration_task(lamp_calibration_task, mocker):
             files = list(task.read(tags=tags))
             assert len(files) == 1
             hdu = fits.open(files[0])[0]
-            np.testing.assert_equal(hdu.data, np.ones((10, 10)))  # Because lamps are normalized
+            np.testing.assert_allclose(hdu.data, np.ones((10, 10)))  # Because lamps are normalized
 
     tags = [
         VispTag.task("LAMP_GAIN"),
@@ -128,6 +141,6 @@ def test_lamp_calibration_task(lamp_calibration_task, mocker):
         with file.open() as f:
             data = json.load(f)
             assert isinstance(data, dict)
-            assert data["total_frames"] == task.count(
+            assert data["total_frames"] == task.scratch.count_all(
                 tags=[VispTag.input(), VispTag.frame(), VispTag.task("LAMP_GAIN")]
             )
