@@ -67,8 +67,8 @@ def test_quality_task(visp_quality_task, mocker):
     """
     Given: A VISPQualityMetrics task
     When: Calling the task instance
-    Then: A single polarimetric noise measurement and datetime is recorded for each map scan for each Stokes Q, U, and V,
-            and a single polarimetric sensitivity measurement and datetime is recorded for each map scan for each Stokes Q, U, and V
+    Then: A single sensitivity measurement and datetime is recorded for each map scan for each Stokes Q, U, and V,
+            and a single noise measurement and datetime is recorded for L1 file for each Stokes Q, U, and V
     """
     mocker.patch(
         "dkist_processing_common.tasks.mixin.metadata_store.GraphQLClient", new=FakeGQLClient
@@ -78,9 +78,10 @@ def test_quality_task(visp_quality_task, mocker):
     task()
     # Then
     num_map_scans = task.constants.num_map_scans
-    polarimetric_noise_files = list(task.read(tags=[Tag.quality("POLARIMETRIC_NOISE")]))
-    assert len(polarimetric_noise_files) == 3
-    for file in polarimetric_noise_files:
+    num_steps = task.constants.num_raster_steps
+    sensitivity_files = list(task.read(tags=[Tag.quality("SENSITIVITY")]))
+    assert len(sensitivity_files) == 4
+    for file in sensitivity_files:
         with file.open() as f:
             data = json.load(f)
             assert isinstance(data, dict)
@@ -90,13 +91,14 @@ def test_quality_task(visp_quality_task, mocker):
                 assert type(data["y_values"][noise]) == float
             assert len(data["x_values"]) == len(data["y_values"]) == num_map_scans
 
-    polarimetric_sens_files = list(task.read(tags=[Tag.quality("POLARIMETRIC_SENSITIVITY")]))
-    assert len(polarimetric_sens_files) == 1
-    with polarimetric_sens_files[0].open() as f:
-        data = json.load(f)
-        assert isinstance(data, dict)
-        for time in range(len(data["x_values"])):
-            assert type(data["x_values"][time]) == str
-        for noise in range(len(data["y_values"])):
-            assert type(data["y_values"][noise]) == float
-        assert len(data["x_values"]) == len(data["y_values"]) == num_map_scans
+    noise_files = list(task.read(tags=[Tag.quality("NOISE")]))
+    assert len(noise_files) == 4
+    for file in noise_files:
+        with file.open() as f:
+            data = json.load(f)
+            assert isinstance(data, dict)
+            for time in range(len(data["x_values"])):
+                assert type(data["x_values"][time]) == str
+            for noise in range(len(data["y_values"])):
+                assert type(data["y_values"][noise]) == float
+            assert len(data["x_values"]) == len(data["y_values"]) == num_map_scans * num_steps
